"""Rulings parsing.
"""
from typing import Generator, Tuple
import importlib.resources
import re
import warnings
import yaml


class Ruling:
    """Simple class representing a ruling."""

    def __init__(self):
        self.cards = []
        self.text = ""
        self.clean_text = ""
        self.links = {}
        self.symbols = []


class RulingReader:
    """Reader to load KRCG YAML rulings files."""

    def __init__(self):
        self.links = yaml.safe_load(
            importlib.resources.files("rulings")
            .joinpath("rulings-links.yaml")
            .read_text("utf-8")
        )

    def __iter__(self):
        """Yield Ruling instances"""
        for card, rulings in yaml.safe_load(
            importlib.resources.files("rulings")
            .joinpath("cards-rulings.yaml")
            .read_text("utf-8")
        ).items():
            for ruling in rulings:
                ret = Ruling()
                ret.cards = [_card_id_name(card)]
                ret.text = ruling
                if not ret.text or not isinstance(ret.text, str):
                    warnings.warn(f"absent or misformed text in '{card}' ruling")
                try:
                    ret.links = dict(self._get_link(ret.text))
                except KeyError:
                    warnings.warn(f"Ruling: link not found for `{card}`")
                    raise
                ret.symbols = list(self._get_symbols(ret.text))
                ret.clean_text = self._clean_text(ret.text)
                yield ret
        for ruling in yaml.safe_load(
            importlib.resources.files("rulings")
            .joinpath("general-rulings.yaml")
            .read_text("utf-8")
        ):
            ret = Ruling()
            ret.cards = [_card_id_name(card) for card in ruling["cards"]]
            ret.text = ruling["ruling"]
            if not ret.text or not isinstance(ret.text, str):
                warnings.warn(
                    f"absent or misformed text in general ruling for {ret.cards}"
                )
            try:
                ret.links = dict(self._get_link(ret.text))
            except KeyError:
                warnings.warn(f"Ruling: link not found for general ruling `{ret.text}`")
                raise
            ret.symbols = list(self._get_symbols(ret.text))
            ret.clean_text = self._clean_text(ret.text)
            yield ret

    def _get_link(self, text: str) -> Generator:
        """Yield (reference, link) tuples from rulink text."""
        references = re.findall(r"\[[a-zA-Z]+\s[0-9-]+\]", text)
        if not references:
            warnings.warn(f"no reference in ruling: {text}")
        for reference in references:
            yield reference, self.links[reference[1:-1]]

    def _get_symbols(self, text: str) -> Generator:
        """Yield symbols from ruling text."""
        symbols = re.findall(r"\[[a-zA-Z ]+\]", text)
        for symbol in symbols:
            yield symbol[1:-1]

    def _clean_text(self, text: str) -> None:
        text = re.subn(r"\[[a-zA-Z]+\s[0-9-]+\]", "", text)[0]
        text = re.subn(r"\[[a-zA-Z ]+\]", "", text)[0]
        return text


def _card_id_name(text: str) -> Tuple[int, str]:
    """Decode (id, name) from YAML "id|name" format."""
    try:
        card_id, card_name = text.split("|")
    except ValueError:
        warnings.warn(f"Bad card ID: {text}")
        raise
    card_id = int(card_id)
    return card_id, card_name



"""Sets (expansions) information
"""
import datetime
from typing import Hashable

from . import utils


class Set(utils.i18nMixin, utils.NamedMixin):
    """A class representing a V:tES Set (expansion)."""

    def __init__(self, **kwargs):
        super().__init__()
        self.id = 0
        self.abbrev = kwargs.get("abbrev", None)
        self.release_date = kwargs.get("release_date", None)
        self.name = kwargs.get("name", None)
        self.company = kwargs.get("abbrev", None)

    def from_vekn(self, data: dict):
        """Load info from VEKN CSV dict."""
        self.id = int(data["Id"])
        self.abbrev = data["Abbrev"]
        self.release_date = (
            datetime.datetime.strptime(data["Release Date"], "%Y%m%d")
            .date()
            .isoformat()
        )
        self.name = data["Full Name"]
        self.company = data["Company"]


class SetMap(dict):
    """A dict of all sets, index by Abbreviation and English name."""

    PROMOS = {
        "Promo-20230531": ["2023 Promo", "2023-05-31"],
        "Promo-20221022": ["2022 Promo", "2022-10-22"],
        "HttBR": ["Heirs to the Blood Reprint", "2018-07-14"],
        "KoTR": ["Keepers of Tradition Reprint", "2018-05-05"],
    }

    def __init__(self):
        super().__init__()
        self.add(Set(abbrev="POD", name="Print on Demand"))
        for abbrev, (name, release_date) in self.PROMOS.items():
            self.add(Set(abbrev=abbrev, name=name, release_date=release_date))

    def add(self, set_: Set) -> None:
        """Add a set to the map."""
        self[set_.abbrev] = set_
        self[set_.name] = set_

    def i18n_set(self, set_: Set) -> None:
        """Add a translation for a set."""
        self[set_.abbrev].i18n_set()


class DefaultSetMap(dict):
    """A default map with no information other than the set abbreviation.

    Can be used to enable card information parsing when no set info is available.
    """

    def __getitem__(self, k: Hashable) -> Set:
        return Set(id=1, abbrev=k, name=k)


#: Use the default set map to parse cards information with no set information available
DEFAULT_SET_MAP = DefaultSetMap()
